--- Draw rectangles with rounded corners with LÖVE (http://love2d.org)
local modname = ...
local M = {}
_G[modname] = M
package.loaded[modname] = M

local ipairs = ipairs
local lg = love.graphics
local PI = math.pi
local sin = math.sin
local cos = math.cos
local unpack = unpack

if _ENV then
    _ENV = M
else
    setfenv(1, M)
end

local TWOPI = PI * 2
local HALFPI = PI * 0.5
local PI32 = PI + HALFPI

function extend(t1, t2)
    for _, v in ipairs(t2) do
        t1[#t1+1] = v
    end
end

--- Return table of coordinates of a circle arc with step resolution
function arc_coords(x, y, radius, start, stop, step)
    step = step or (TWOPI * 0.01)
    if start > stop then
        step = -step
    end

    local coords = {}
    for angle = start, stop, step do
        coords[#coords+1] = x + cos(angle) * radius
        coords[#coords+1] = y + sin(angle) * radius
    end
    return coords
end

--- Draw rectangle with rounded corners to a canvas and return it
function draw_rect(mode, w, h, radius, color, bgcolor, linewidth, aa, step)
    radius = radius or 0
    linewidth = linewidth or 1
    local hlw = linewidth * 0.5
    local x1 = radius + hlw
    local y1 = radius + hlw
    local x2 = w - radius - hlw
    local y2 = h - radius - hlw

    if radius > 0 then
        coords = arc_coords(x1, y1, radius, PI, PI32, step)
        extend(coords, arc_coords(x2, y1, radius, PI32, TWOPI, step))
        extend(coords, arc_coords(x2, y2, radius, 0, HALFPI, step))
        extend(coords, arc_coords(x1, y2, radius, HALFPI, PI, step))
    else
        coords = {x1, hlw, x2, hlw, w - hlw, y2, x1, h - hlw}
    end

    local canvas = lg.newCanvas(w + linewidth, h + linewidth, "normal", aa)
    lg.setCanvas(canvas)
        canvas:clear()
        lg.setBackgroundColor({255, 255, 255, 0})
        lg.setBlendMode('alpha')
        lg.setLineStyle("smooth")

        if bgcolor then
            lg.setLineWidth(0)
            lg.setColor(unpack(bgcolor))
            lg.polygon("fill", coords)
        end

        lg.setColor(unpack(color))
        lg.setLineWidth(linewidth)

        lg.polygon(mode, coords)
    lg.setCanvas()
    return canvas
end
